/*
 * DemosceneMusic - EditArtistPage.java - Copyright © 2012 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.demoscenemusic.page.admin;

import java.util.Collection;

import net.pterodactylus.demoscenemusic.core.Core;
import net.pterodactylus.demoscenemusic.data.Artist;
import net.pterodactylus.demoscenemusic.data.Group;
import net.pterodactylus.demoscenemusic.page.ServletRequest;
import net.pterodactylus.util.database.DatabaseException;
import net.pterodactylus.util.template.Template;
import net.pterodactylus.util.template.TemplateContext;
import net.pterodactylus.util.template.TemplateContextFactory;
import net.pterodactylus.util.web.Method;
import net.pterodactylus.util.web.RedirectException;

/**
 * Page that lets the user edit an artist.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class EditArtistPage extends AdminBasePage {

	/**
	 * Creates a new “edit artist” page.
	 *
	 * @param core
	 *            The core
	 * @param templateContextFactory
	 *            The template context factory
	 * @param template
	 *            The template to render
	 */
	public EditArtistPage(Core core, TemplateContextFactory templateContextFactory, Template template) {
		super(core, templateContextFactory, template, "admin.edit-artist");
	}

	//
	// BASEPAGE METHODS
	//

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void processTemplate(TemplateContext templateContext, ServletRequest request) throws RedirectException {
		super.processTemplate(templateContext, request);
		String id = request.getServletRequest().getParameter("id");
		try {
			Artist artist = getCore().getDataManager().getArtistById(id);
			/* TODO - check for null. */
			templateContext.set("artist", artist);
			if (request.getMethod() == Method.POST) {
				String name = request.getServletRequest().getParameter("name");
				if ((name != null) && (name.trim().length() > 0)) {
					artist.setName(name);
				}

				/* process property changes. */
				processProperties(request, artist);

				/* check for new group. */
				if ("true".equals(request.getServletRequest().getParameter("add-group"))) {
					String groupId = request.getServletRequest().getParameter("group");
					Group group = getCore().getDataManager().getGroupById(groupId);
					Collection<Group> groups = artist.getGroups();
					groups.add(group);
					artist.setGroups(groups);
				}

				/* check for deleted groups. */
				if ("true".equals(request.getServletRequest().getParameter("delete-group"))) {
					String groupId = request.getServletRequest().getParameter("group");
					Group group = getCore().getDataManager().getGroupById(groupId);
					Collection<Group> groups = artist.getGroups();
					groups.remove(group);
					artist.setGroups(groups);
				}

				getCore().getDataManager().saveArtist(artist);
				throw new RedirectException("admin.edit-artist?id=" + artist.getId());
			}
		} catch (DatabaseException de1) {
			throw new RuntimeException("Could not load artist.", de1);
		}
	}

}
