/*
 * DemosceneMusic - EditTrackPage.java - Copyright © 2012 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.demoscenemusic.page.admin;

import java.util.Collection;
import java.util.List;

import net.pterodactylus.demoscenemusic.core.Core;
import net.pterodactylus.demoscenemusic.data.Artist;
import net.pterodactylus.demoscenemusic.data.Party;
import net.pterodactylus.demoscenemusic.data.Track;
import net.pterodactylus.demoscenemusic.page.ServletRequest;
import net.pterodactylus.util.database.DatabaseException;
import net.pterodactylus.util.template.Template;
import net.pterodactylus.util.template.TemplateContext;
import net.pterodactylus.util.template.TemplateContextFactory;
import net.pterodactylus.util.web.Method;
import net.pterodactylus.util.web.RedirectException;

/**
 * Page that lets the user edit a track.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class EditTrackPage extends AdminBasePage {

	/**
	 * Creates a new edit-track page.
	 *
	 * @param core
	 *            The core
	 * @param templateContextFactory
	 *            The template contex factory
	 * @param template
	 *            The template to render
	 */
	public EditTrackPage(Core core, TemplateContextFactory templateContextFactory, Template template) {
		super(core, templateContextFactory, template, "admin.edit-track");
	}

	//
	// BASEPAGE METHODS
	//

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void processTemplate(TemplateContext templateContext, ServletRequest request) throws RedirectException {
		super.processTemplate(templateContext, request);
		String trackId = request.getServletRequest().getParameter("id");
		try {
			Track track = getCore().getDataManager().getTrackById(trackId);
			if (track == null) {
				templateContext.set("error", "no-track-given");
				return;
			}

			templateContext.set("track", track);
			if (request.getMethod() == Method.POST) {

				String partyId = request.getServletRequest().getParameter("party");
				Party party = getCore().getDataManager().getPartyById(partyId);

				String artistId = request.getServletRequest().getParameter("artist");
				Artist artist = getCore().getDataManager().getArtistById(artistId);

				/* check if a party should be removed. */
				if ("true".equals(request.getServletRequest().getParameter("delete-party"))) {
					Collection<Party> parties = track.getParties();
					parties.remove(party);
					track.setParties(parties);
				}

				/* check if a party should be added. */
				if ("true".equals(request.getServletRequest().getParameter("add-party"))) {
					Collection<Party> parties = track.getParties();
					parties.add(party);
					track.setParties(parties);
				}

				if ("true".equals(request.getServletRequest().getParameter("delete-artist"))) {
					List<Artist> artists = track.getArtists();
					artists.remove(artist);
					track.setArtists(artists);
				}

				if ("true".equals(request.getServletRequest().getParameter("move-up"))) {
					List<Artist> artists = track.getArtists();
					int index = artists.indexOf(artist);
					if (index > 0) {
						artists.remove(artist);
						artists.add(index - 1, artist);
					}
					track.setArtists(artists);
				}

				if ("true".equals(request.getServletRequest().getParameter("move-down"))) {
					List<Artist> artists = track.getArtists();
					int index = artists.indexOf(artist);
					if (index < (artists.size() - 1)) {
						artists.remove(artist);
						artists.add(index + 1, artist);
					}
					track.setArtists(artists);
				}

				if ("true".equals(request.getServletRequest().getParameter("add-artist"))) {
					List<Artist> artists = track.getArtists();
					artists.add(artist);
					track.setArtists(artists);
				}

				String name = request.getServletRequest().getParameter("name");
				if ((name != null) && (name.trim().length() > 0)) {
					track.setName(name);
				}
				String remix = request.getServletRequest().getParameter("remix");
				if ((remix != null) && (remix.trim().length() > 0)) {
					track.setRemix(remix);
				}

				/* process properties. */
				processProperties(request, track);

				getCore().getDataManager().saveTrack(track);
				throw new RedirectException("admin.edit-track?id=" + track.getId());
			}
		} catch (DatabaseException de1) {
			throw new RuntimeException("Could not edit track.", de1);
		}
	}

}
